"""
Simple API for contacts management
"""

from typing import List
from fastapi import APIRouter, Depends, HTTPException, Body
from sqlalchemy.orm import Session
from . import crud, schemas
from toardb.utils.database import ToarDbSession, get_db

router = APIRouter()

# plain views to post and get contacts

# 1. persons

#get all entries of table persons
@router.get('/contacts/persons/', response_model=List[schemas.Person])
def get_all_persons(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    persons = crud.get_all_persons(db, skip=skip, limit=limit)
    return persons

#get all metadata of one person
@router.get('/contacts/persons/id/{person_id}', response_model=schemas.Person)
def get_person(person_id: int, db: Session = Depends(get_db)):
    db_person = crud.get_person(db, person_id=person_id)
    if db_person is None:
        raise HTTPException(status_code=404, detail="Person not found.")
    return db_person

@router.get('/contacts/persons/{name}', response_model=schemas.Person)
def get_person(name: str, db: Session = Depends(get_db)):
    db_person = crud.get_person_by_name(db, name=name)
    if db_person is None:
        raise HTTPException(status_code=404, detail="Person not found.")
    return db_person

@router.post('/contacts/persons/', response_model=schemas.Person)
def create_person(person: schemas.PersonCreate = Body(..., embed = True), db: Session = Depends(get_db)):
    db_person = crud.get_person_by_name(db, name=person.name)
    if db_person:
        raise HTTPException(status_code=400, detail="Person already registered.")
    return crud.create_person(db=db, person=person)


# 2. organisations

#get all entries of table organisations
@router.get('/contacts/organisations/', response_model=List[schemas.Organisation])
def get_all_organisations(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    organisations = crud.get_all_organisations(db, skip=skip, limit=limit)
    return organisations

#get all metadata of one organisation
@router.get('/contacts/organisations/id/{organisation_id}', response_model=schemas.Organisation)
def get_organisation(organisation_id: int, db: Session = Depends(get_db)):
    db_organisation = crud.get_organisation (db, organisation_id=organisation_id)
    if db_organisation is None:
        raise HTTPException(status_code=404, detail="Organisation not found.")
    return db_organisation

@router.get('/contacts/organisations/{name}', response_model=schemas.Organisation)
def get_organisation(name: str, db: Session = Depends(get_db)):
    db_organisation = crud.get_organisation_by_name(db, name=name)
    if db_organisation is None:
        raise HTTPException(status_code=404, detail="Organisation not found.")
    return db_organisation

@router.post('/contacts/organisations/', response_model=schemas.Organisation)
def create_organisation(organisation: schemas.OrganisationCreate = Body(..., embed = True), db: Session = Depends(get_db)):
    db_organisation = crud.get_organisation_by_name(db, name=organisation.name)
    if db_organisation:
        raise HTTPException(status_code=400, detail="Organisation already registered.")
    return crud.create_organisation(db=db, organisation=organisation)

# 3. combined (person + organisation)
#get all entries of table contacts
@router.get('/contacts/', response_model=List[schemas.Contact])
def get_all_contacts(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    contacts = crud.get_all_contacts(db, skip=skip, limit=limit)
    return contacts

#get a special entry of table contacts (given by ID)
@router.get('/contacts/id/{contact_id}', response_model=schemas.Contact)
def get_all_contacts(contact_id: int, db: Session = Depends(get_db)):
    contact = crud.get_contact(db, contact_id=contact_id)
    return contact

